/*
	IRCaccount.cpp - Kopete IRC Protocol

	Copyright (c) 2009      by Volker Härtel		 <folkyvolk@gmx.de>
    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This library is free software; you can redistribute it and/or         *
    * modify it under the terms of the GNU General Public                   *
    * License as published by the Free Software Foundation; either          *
    * version 2 of the License, or (at your option) any later version.      *
    *                                                                       *
    *************************************************************************
*/

/*
TODO:
- contact sorting, faster memberlist
- status changes
*/

#include <ircclient-qt/ircsession.h>


#include "ircaccount.h"

#include <kaction.h>
#include <kdebug.h>
#include <klocale.h>
#include <kactionmenu.h>
#include <kmenu.h>
#include <kicon.h>
#include <kinputdialog.h>
#include <kmessagebox.h>
#include <kconfig.h>
#include <kcharsets.h>

#include "kopeteuiglobal.h"
#include "kopeteview.h"
#include "kopetemetacontact.h"
#include "kopetecontactlist.h"
#include "kopetechatsession.h"
#include "kopetechatsessionmanager.h"

#include "irccontact.h"
#include "ircprotocol.h"
#include "ircnetwork.h"

#include "qtimer.h"
#include "qmutex.h"
#include "qcoreapplication.h"

class IRCAccount::Private
{
public:

	int currentHost;

//	QMutex mutex;

	Irc::Session *m_ircSession;

	IRCAccount *parent;

	IRCContact *server;

	IRCContact *myself;

	QHash<QString, IRCContact*> allContacts;

	~Private(){
		foreach(IRCContact *c,allContacts){
			if (c!=myself){
				c->metaContact()->deleteLater();
				c->deleteLater();
			}
		}
		m_ircSession->deleteLater();
	}

	IRCContact* findContact(QString name, bool create = false, IRCContact::Type type = IRCContact::User){
		if (allContacts.contains(name) || !create)
			return allContacts.value(name);
//		kDebug(14120) << "create new " << endl;
		Kopete::MetaContact *m = new Kopete::MetaContact;
		m->setTemporary(true);
		IRCContact * c = new IRCContact( parent, name,m , type);
		addContact(c);
		return c;
	}

	void addContact(IRCContact *contact){
		allContacts.insert(contact->nickName(), contact);
	}


	void removeContact(IRCContact *contact, QString reason = QString()){
//		kDebug(14120) << "remove contact " << contact->nickName() << endl;
		if (contact->type != IRCContact::Channel)
			foreach (Kopete::ChatSession *session,Kopete::ChatSessionManager::self()->sessions()){
				if (session->members().contains(contact)){
//					kDebug(14120) << "remove contact from session" << session->displayName() << endl;
					session->removeContact(contact, reason);
				} else {
//					kDebug(14120) << "contact is not in session" << session->displayName() << endl;
				}
			}

		allContacts.remove(contact->nickName());
		contact->metaContact()->deleteLater();
		contact->deleteLater();
	}

	void displayAction(QString msg,bool activeView = true){
			Kopete::Message mes(server,myself);
			mes.setDirection(Kopete::Message::Inbound);
			mes.setPlainBody(msg);
			mes.setType(Kopete::Message::TypeAction);
			mes.setImportance(Kopete::Message::Low);
			if (activeView)
				Kopete::ChatSessionManager::self()->activeView()->msgManager()->appendMessage(mes);
			else
				server->manager(Kopete::Contact::CanCreate)->appendMessage(mes);
	}

};

IRCAccount::IRCAccount( IRCProtocol *parent, const QString& accountID )
: Kopete::PasswordedAccount( parent, accountID, true ), d(new Private)
{

	d->parent = this;
	d->currentHost = 0;
	const QString serverName = accountId();


	nick = configGroup()->readEntry("NickName",QString());
	nick2 = configGroup()->readEntry("AlternativeNickName",QString());
	realname = configGroup()->readEntry("RealName",QString());
	network = configGroup()->readEntry("NetworkName",QString());
	encoding = configGroup()->readEntry("Codec",QString());
	connectCommands = configGroup()->readEntry("Commands",QStringList());

//	kDebug(14120) << "constructor account id="<< accountID << ", have read from config: nick="<<nick<<
//			", nick2="<<nick2<<", realname="<<realname;


	d->m_ircSession = new Irc::Session(this);
	d->m_ircSession->setOptions(Irc::Session::StripNicks);
	d->m_ircSession->connectSlotsByName(this);


	// Init the myself contact
	d->myself = new IRCContact( this, nick,  Kopete::ContactList::self()->myself() );
	d->myself->setOnlineStatus( IRCProtocol::protocol()->StatusOffline );
	setMyself( d->myself );

	d->server = new IRCContact( this, accountId(), new  Kopete::MetaContact(),IRCContact::Server);

	d->addContact(d->server);
	d->addContact(static_cast<IRCContact*>(myself()));

//	QObject::connect(this,SIGNAL(handleNames(QString,QStringList)),this,SLOT(slotHandleNames(QString,QStringList)));



}

IRCAccount::~IRCAccount()
{
	delete d;
}

void IRCAccount::on_connected()
{

	kDebug(14120) << "on connected" << endl;

	foreach (IRCContact *c,d->allContacts){
		if (c->type == IRCContact::Channel){
			d->m_ircSession->cmdJoin(c->nickName());
			c->setOnlineStatus( IRCProtocol::protocol()->StatusChannel);
		} else {
			c->setOnlineStatus( IRCProtocol::protocol()->StatusOnline);
		}


	}

//	myself()->setOnlineStatus( IRCProtocol::protocol()->StatusOnline);
//	d->server->setOnlineStatus(IRCProtocol::protocol()->StatusOnline);
	d->server->startChat();

	foreach (QString cmd,connectCommands){
		d->m_ircSession->sendRaw(cmd);
	}

}



void IRCAccount::on_disconnected ()
{
	kDebug(14120) << "ondisconnected" << endl;

	//clean up: delete all contacts but myself/server and those with chatsessions
	QMutableHashIterator<QString,IRCContact* > it(d->allContacts);
	while (it.hasNext()){
		IRCContact *c = it.next().value();
		if (c!=d->myself && c!=d->server && !c->manager(Kopete::Contact::CannotCreate)){
			it.remove();
			foreach (Kopete::ChatSession *session,Kopete::ChatSessionManager::self()->sessions()){
				session->removeContact(c, QString(),Qt::PlainText,true);
			}
			c->metaContact()->deleteLater();
			c->deleteLater();
		}
	}
	foreach (IRCContact *c,d->allContacts)
		c->setOnlineStatus(IRCProtocol::protocol()->StatusOffline);

}


void IRCAccount::on_msgMessageReceived (const QString &origin, const QString &receiver, const QString &message)
{
	kDebug(14120) << "onmessagreceived from " << origin << " to " << receiver << ": " << message <<endl;
	Kopete::Contact *from = d->findContact(origin);
	Kopete::Contact *to = d->findContact(receiver);
	if (to && from){
		kDebug(14120) << "have to and from set, from-nick="<<from->nickName()
				<<", from-meta-name="<< from->metaContact()->displayName() << endl;

		if (to==myself()){
			kDebug(14120) << "message to myself" << endl;
			Kopete::Message mes(from, to);
			mes.setPlainBody(message);
			mes.setDirection(Kopete::Message::Inbound);
			mes.setImportance(Kopete::Message::Normal);
			from->manager(Kopete::Contact::CanCreate)->appendMessage(mes);
		} else if (from==myself()) {
			kDebug(14120) << "message from myself" << endl;
			Kopete::Message mes(from, to);
			mes.setPlainBody(message);
			mes.setDirection(Kopete::Message::Outbound);
			mes.setImportance(Kopete::Message::Low);
			to->manager(Kopete::Contact::CanCreate)->appendMessage(mes);
		} else {
			kDebug(14120) << "message without myself" << endl;
			Kopete::Message mes(from, to);
			mes.setPlainBody(message);
			mes.setDirection(Kopete::Message::Inbound);
			mes.setImportance(Kopete::Message::Low);
			to->manager(Kopete::Contact::CanCreate)->appendMessage(mes);
		}



	} else {
		kDebug(14120) << "have NOT to and from set" << endl;
	}
}



void IRCAccount::on_msgJoined (const QString &origin, const QString &channel)
{
	kDebug(14120) << "onjoined channel: origin = " << origin << ", channel = " << channel << endl;

	//cibaer78!n=cibaer78@82.83.148.178

	IRCContact *c = d->findContact(origin, true);

	if (c == myself())
	{
		kDebug(14120) << "join channel, create channel, start chat" << endl;
		IRCContact *ch = d->findContact(channel,true, IRCContact::Channel);
		ch->manager(Kopete::Contact::CanCreate)->setDisplayName(channel);
		ch->startChat();

	}
	else
	{
		kDebug(14120) << "let user join channel" << endl;
		Kopete::Contact *ch = d->findContact(channel);
		if (ch){
			ch->manager(Kopete::Contact::CanCreate)->addContact(c);
		}
	}
}

void IRCAccount::on_msgParted (const QString &origin, const QString &channel, const QString &message)
{
	kDebug(14120) << "onparted: origin = " << origin << ", channel = " << channel << endl;

	IRCContact *c = d->findContact(origin);

	if (c)
	{
		IRCContact *ch = d->findContact(channel);
		if (c != myself())
		{
			if (ch)
			{
				ch->manager(Kopete::Contact::CanCreate)->removeContact(c, message);
			}
		} else {
			d->removeContact(ch);
		}
	}
}


void IRCAccount::on_msgQuit (const QString &origin, const QString &message)
{
	kDebug(14120) << "onquit, origin = " << origin  << endl;
	IRCContact *c = d->findContact(origin);
	if (c)
		d->removeContact(c, message);
}

void IRCAccount::on_msgNickChanged (const QString &origin, const QString &nick)
{
	kDebug(14120) << "onnickchange origin="<< origin << ", newnick="<<nick << endl;
	IRCContact *c = d->findContact(origin);
	if (c){
		kDebug(14120) << "nick change success" << endl;
		d->allContacts.remove(c->nickName());
		c->setNickName(nick);
		d->addContact(c);
	}

}

void IRCAccount::on_msgModeChanged (const QString &origin, const QString &receiver, const QString &mode, const QString &args)
{
	kDebug(14120) << "onmodehcnage" << endl;
}

void IRCAccount::on_msgTopicChanged (const QString &origin, const QString &channel, const QString &topic)
{
	kDebug(14120) << "ontopicchange" << endl;

			{
			IRCContact *ch = d->findContact(channel);
			if (ch){
				kDebug(14120) << "set topic" << endl;
				Kopete::Message mes(ch,myself());
				mes.setDirection(Kopete::Message::Inbound);
				mes.setPlainBody(i18n("%1 has changed the topic to: %2").arg(origin, topic));
				mes.setImportance(Kopete::Message::Low);
				ch->manager(Kopete::Contact::CanCreate)->appendMessage(mes);
			} else
				kDebug(14120) << "channel for topic not found" << endl;
		}
}

void IRCAccount::on_msgInvited (const QString &origin, const QString &receiver, const QString &channel)
{
	kDebug(14120) << "oninviteed" << endl;
}

void IRCAccount::on_msgKicked (const QString &origin, const QString &channel, const QString &nick, const QString &message)
{
	kDebug(14120) << "onkicked, redirect to parted" << endl;
	on_msgParted(nick, channel, message);

}


void IRCAccount::on_msgNoticeReceived (const QString &origin, const QString &receiver, const QString &notice)
{
	kDebug(14120) << "onnoticereceived from " << origin << " to " << receiver << ": " << notice <<endl;
}

void IRCAccount::on_msgCtcpRequestReceived (const QString &origin, const QString &request)
{
	kDebug(14120) << "onctcprequest" << endl;
}

void IRCAccount::on_msgCtcpReplyReceived (const QString &origin, const QString &reply)
{
	kDebug(14120) << "onctcpreply" << endl;
}

void IRCAccount::on_msgCtcpActionReceived (const QString &origin, const QString &receiver, const QString &action)
{
	kDebug(14120) << "onctpcaction" << endl;
}

void IRCAccount::on_msgNumericMessageReceived (const QString &origin, uint code, const QStringList &params)
{
	kDebug(14120) << "onnumericmessage from " << origin << ", code=" << code << "params= " << params << endl;
	switch (code)
	{
		case 353:
			{
				QString channel = params.value(2);
				QStringList members = params.value(3).split(" ");
				slotHandleNames(channel, members);
			}
			break;
		case 366://end of names list, TODO: better put this in onjoined, when kopete-membersmodel is optimized
			{
				Kopete::Contact *channel = d->findContact(params.value(1));
				if (channel){
//					channel->startChat();
//					remove all offline contacts (dead ones in case of reconnect):
//					foreach (Kopete::Contact *c,channel->manager()->members())
//						if (c->onlineStatus()==IRCProtocol::protocol()->StatusOffline)
//							channel->manager()->removeContact(c);
				}
			}

			break;
		case 332: //topic
			{
				IRCContact *ch = d->findContact(params.value(1));
				if (ch){
					kDebug(14120) << "set topic" << endl;
					Kopete::Message mes(ch,myself());
					mes.setDirection(Kopete::Message::Inbound);
					mes.setPlainBody(params.value(2));
					mes.setImportance(Kopete::Message::Low);
					ch->manager(Kopete::Contact::CanCreate)->appendMessage(mes);
				} else
					kDebug(14120) << "channel for topic not found" << endl;
			}
			break;
		case 333: //topic
			{
				IRCContact *ch = d->findContact(params.value(1));
				if (ch){
					QDateTime d;
					d.setTime_t( params.value(3).toLong() );
					QString time = KGlobal::locale()->formatDateTime(d, KLocale::ShortDate, true);
					kDebug(14120) << "set topic creator" << endl;
					Kopete::Message mes(ch,myself());
					mes.setDirection(Kopete::Message::Inbound);
					mes.setPlainBody(i18n("The topic was changed at %1 by %2").arg(time, params.value(2)));
					mes.setImportance(Kopete::Message::Low);
					ch->manager(Kopete::Contact::CanCreate)->appendMessage(mes);
				} else
					kDebug(14120) << "channel for topic not found" << endl;
			}
			break;
		//server connect replies
		case 1:
		case 2:
		case 3:
		case 251:
		case 252:
		case 254:
		case 255:
		case 265:
		case 266:
		case 375:
		case 372:
			{
				if (code==1)
					d->server->manager(Kopete::Contact::CanCreate)->setDisplayName(origin);
				QStringList msg = params;
				msg.takeFirst();
				d->displayAction(msg.join(" "),false);
				QCoreApplication::processEvents();
			}
			break;
		case 381://you are now operator
			{
				d->myself->setIRCStatus((IRCProtocol::IRCStatus) (d->myself->status | IRCProtocol::Operator));
			}
			break;
		case 433: //nickname in use, try alternative nick, if this is the registered nick
			{
				if (params[1]==nick){
					on_msgNickChanged(nick, nick2);
					d->m_ircSession->setNick(nick2);
				}
			}
			break;
		default:
			{
				QStringList msg = params;
				msg.takeFirst();
				d->displayAction(msg.join(" "));
			}
	}
}


void IRCAccount::slotHandleNames(const QString &channelName, const QStringList &names)
{
	kDebug(14120) << "add contacts to channel " << channelName << endl;

	Kopete::Contact *channel = d->findContact(channelName);
	if (channel){
//		kDebug(14120) << "have found channel" << endl;
		foreach (QString name,names)
		{
//			kDebug(14120) << "try to add contact " << name << " to channel " << channelName << endl;

			int status = IRCProtocol::Online;

			while (name.size()>1){
				if (name.startsWith("@"))
					status |= IRCProtocol::Operator;
				else if (name.startsWith("+"))
					status |= IRCProtocol::Voiced;
				else
					break;
				name = name.mid(1);
			}
			IRCContact *c = d->findContact(name, true);

			QCoreApplication::processEvents();

			if (c != myself())
			{
//				QTimer::singleShot(0,this,SIGNAL
				channel->manager(Kopete::Contact::CanCreate)->addContact(c,true);
			}
			if (status!=IRCProtocol::Online)
				channel->manager(Kopete::Contact::CanCreate)->setContactOnlineStatus(c, IRCProtocol::protocol()->statusLookup((IRCProtocol::IRCStatus)status));

			QCoreApplication::processEvents();
		}
	}
}

void IRCAccount::on_msgUnknownMessageReceived (const QString &origin, const QStringList &params)
{
	kDebug(14120) << "onunknownmssage" << endl;
}



void IRCAccount::fillActionMenu( KActionMenu *actionMenu )
{
//	kDebug(14120) << "fillacountmenu" << endl;
	Kopete::Account::fillActionMenu( actionMenu );

	actionMenu->addSeparator();

	KAction *action;

	action = new KAction (KIcon("irc_channel"), i18n ("Join channel..."), actionMenu );
	QObject::connect( action, SIGNAL(triggered(bool)), this, SLOT(slotJoinChannel()) );
	actionMenu->addAction(action);
	action->setEnabled( isConnected() );
}

void IRCAccount::slotJoinChannel()
{
	kDebug(14120) << "join chanelslot" << endl;
	if (!isConnected())
		return;

	QStringList chans = configGroup()->readEntry( "Recent Channel list", QStringList() );

	QString channelName=KInputDialog::getText( i18n( "Join Channel" ),
			i18n("Please enter the name of the channel you want to join:"),
			QString(), 0,
			Kopete::UI::Global::mainWidget(),
			0, QString(), 0, chans
		);

	if ( !channelName.isNull() )
	{
		kDebug( 14120 )<<"joining channel"<<channelName;
		chans.prepend( channelName );
		chans.removeDuplicates();
		while (chans.size()>20)
			chans.removeLast();
		configGroup()->writeEntry( "Recent Channel list", chans );

		if (d->m_ircSession != 0 && d->m_ircSession->isConnected()){
			d->m_ircSession->cmdJoin(channelName);
		}
	}
}

bool IRCAccount::createContact(const QString& contactId, Kopete::MetaContact* parentContact)
{
	kDebug(14120) << "create contact" << contactId << endl;
	IRCContact* newContact = new IRCContact( this, contactId,  parentContact );
	return newContact != 0L;
}


void IRCAccount::setOnlineStatus(const Kopete::OnlineStatus& status, const Kopete::StatusMessage &reason, const OnlineStatusOptions& options)
{
	qWarning ( ) << "setonlinesttaus" << endl;
	if ( status.status() == Kopete::OnlineStatus::Online &&
			myself()->onlineStatus().status() == Kopete::OnlineStatus::Offline )
		slotGoOnline();
	else if ( status.status() == Kopete::OnlineStatus::Offline )
		slotGoOffline();
}

void IRCAccount::setStatusMessage(const Kopete::StatusMessage& statusMessage)
{
	Q_UNUSED(statusMessage);
	/* Not used in IRC */
}

void IRCAccount::connectWithPassword( const QString &password  )
{
	kDebug(14120) << "connect" << endl;

	d->m_ircSession->setNick(nick);
	d->m_ircSession->setAutoReconnectDelay(5);
	d->m_ircSession->setRealName(realname);
	KCharsets *c = KGlobal::charsets();
	QTextCodec *codec = c->codecForName(encoding);
	if (codec)
		d->m_ircSession->setEncoding(encoding);


	if (!password.isEmpty())
		d->m_ircSession->setPassword(password);

	IRC::Network network = IRC::Networks::self()->network(this->network);

	if (!network.hosts.isEmpty()){
		IRC::Host host = network.hosts.first();
		d->m_ircSession->connectToServer(host.host, host.port);
		myself()->setOnlineStatus( Kopete::OnlineStatus::Connecting );
	} else {
		KMessageBox::queuedMessageBox(
			Kopete::UI::Global::mainWidget(), KMessageBox::Error,
		i18n("<qt>Kopete could not connect to any of the servers in the network associated with this account (<b>%1</b>). Please try again later.</qt>").arg(network.name),
		i18n("Network is Unavailable"), 0 );
	}
}

void IRCAccount::disconnect()
{
	kDebug(14120) << "disconnect" << endl;

	if (d->m_ircSession == 0)
		return;

	//call disconnect even if disconnection fails:
	on_disconnected();

	d->m_ircSession->disconnectFromServer();
}



void IRCAccount::slotGoOnline ()
{
	kDebug(14120) << "slotgoonline" << endl;

	if (!isConnected ())
		connect ();
	else
		myself()->setOnlineStatus( IRCProtocol::protocol()->StatusOnline );
}



void IRCAccount::slotGoOffline ()
{
	kDebug(14120) << "slotgooffline" << endl;

	if (isConnected ())
		disconnect ();
}

void IRCAccount::sendMessage( Kopete::Message &message, Kopete::ChatSession* session )
{

	kDebug(14120) << "sendmessage from=" << message.from()->contactId()<< " to="<< message.to().first()->contactId() << endl;

	Kopete::Contact* from = const_cast<Kopete::Contact*>(message.from());



	if (message.plainBody().startsWith("/"))
		d->m_ircSession->sendRaw(message.plainBody().mid(1));
	else {
		kDebug(14120) << "sendmessage to " << message.to().first()->nickName() << endl;
		d->m_ircSession->cmdMessage(message.to().first()->nickName(),message.plainBody());
	}

}

void IRCAccount::slotChatSessionClose(Kopete::ChatSession* session, Kopete::Contact *c)
{
	kDebug(14120) << "slot chatsessionclose" << endl;
	if (d->m_ircSession->isConnected() && c)
		d->m_ircSession->cmdPart(c->nickName());

}


void IRCAccount::storeSettings()
{

	configGroup()->writeEntry("NickName",nick);
	configGroup()->writeEntry("AlternativeNickName",nick2);
	configGroup()->writeEntry("RealName",realname);
	configGroup()->writeEntry("Codec",encoding);
	configGroup()->writeEntry("NetworkName",network);
	configGroup()->writeEntry("Commands",connectCommands);
	configGroup()->writeEntry("PreferSSL", preferssl);

}

#include "ircaccount.moc"

