/*
	IRCaccount.h - Kopete IRC Protocol

	Copyright (c) 2009      by Volker Härtel		 <folkyvolk@gmx.de>
    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This library is free software; you can redistribute it and/or         *
    * modify it under the terms of the GNU General Public                   *
    * License as published by the Free Software Foundation; either          *
    * version 2 of the License, or (at your option) any later version.      *
    *                                                                       *
    *************************************************************************
*/

#ifndef IRCACCOUNT_H
#define IRCACCOUNT_H



#include <kopetepasswordedaccount.h>
#include "kopetemessage.h"


class KActionMenu;
namespace Kopete 
{ 
	class Contact;
	class MetaContact;
	class StatusMessage;
}

class IRCProtocol;

/**
 * This represents an account connected to the IRC
 * @author Volker Härtel
*/
class IRCAccount : public Kopete::PasswordedAccount
{
	Q_OBJECT
public:
	IRCAccount( IRCProtocol *parent, const QString& accountID );

	~IRCAccount();

	/**
	 * Construct the context menu used for the status bar icon
	 */
	virtual void fillActionMenu( KActionMenu *actionMenu );

	/**
	 * Creates a protocol specific Kopete::Contact subclass and adds it to the supplie
	 * Kopete::MetaContact
	 */
	virtual bool createContact(const QString& contactId, Kopete::MetaContact* parentContact);

	/**
	 * Called when Kopete status is changed globally
	 */
	virtual void setOnlineStatus(const Kopete::OnlineStatus& status , const Kopete::StatusMessage &reason = Kopete::StatusMessage(),
	                             const OnlineStatusOptions& options = None);

	virtual void setStatusMessage(const Kopete::StatusMessage& statusMessage);

	/**
	 * 'Connect' to the IRC server.  Only sets myself() online.
	 */
	virtual void connectWithPassword(const QString &password  );

	/**
	 * Disconnect from the server.  Only sets myself() offline.
	 */
	virtual void disconnect();

	/**
	 * Store account settings in konfiggroup
	 */
	void storeSettings();


	QString nick;

	QString nick2;

	QString realname;

	QString network;

	QByteArray encoding;

	QStringList connectCommands;

	bool preferssl;

public slots:

	void sendMessage( Kopete::Message &message, Kopete::ChatSession* );
	void slotChatSessionClose(Kopete::ChatSession* session, Kopete::Contact *c);

	void on_connected();
	void on_disconnected ();
	void on_msgJoined (const QString &origin, const QString &channel);
	void on_msgParted (const QString &origin, const QString &channel, const QString &message);
	void on_msgQuit (const QString &origin, const QString &message);
	void on_msgNickChanged (const QString &origin, const QString &nick);
	void on_msgModeChanged (const QString &origin, const QString &receiver, const QString &mode, const QString &args);
	void on_msgTopicChanged (const QString &origin, const QString &channel, const QString &topic);
	void on_msgInvited (const QString &origin, const QString &receiver, const QString &channel);
	void on_msgKicked (const QString &origin, const QString &channel, const QString &nick, const QString &message);
	void on_msgMessageReceived (const QString &origin, const QString &receiver, const QString &message);
	void on_msgNoticeReceived (const QString &origin, const QString &receiver, const QString &notice);
	void on_msgCtcpRequestReceived (const QString &origin, const QString &request);
	void on_msgCtcpReplyReceived (const QString &origin, const QString &reply);
	void on_msgCtcpActionReceived (const QString &origin, const QString &receiver, const QString &action);
	void on_msgNumericMessageReceived (const QString &origin, uint code, const QStringList &params);
	void on_msgUnknownMessageReceived (const QString &origin, const QStringList &params);

	void slotHandleNames(const QString &origin, const QStringList &params);

protected:


protected slots:
	/**
	 * Change the account's status.  Called by KActions and internally.
	 */
	void slotGoOnline();

	/**
	 * Change the account's status.  Called by KActions and internally.
	 */
	void slotGoOffline();

	void slotJoinChannel();

private:
	class Private;

	Private * const d;

signals:
	void handleNames(const QString &channelName, const QStringList &names);

};

#endif
