/*
	IRCcontact.cpp - Kopete IRC Protocol

	Copyright (c) 2009      by Volker Härtel		 <folkyvolk@gmx.de>
    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This library is free software; you can redistribute it and/or         *
    * modify it under the terms of the GNU General Public                   *
    * License as published by the Free Software Foundation; either          *
    * version 2 of the License, or (at your option) any later version.      *
    *                                                                       *
    *************************************************************************
*/

#include "irccontact.h"

#include <QList>

#include <kaction.h>
#include <kdebug.h>
#include <klocale.h>

#include "kopeteaccount.h"
#include "kopetechatsessionmanager.h"
#include "kopetemetacontact.h"

#include "ircaccount.h"
#include "ircprotocol.h"

IRCContact::IRCContact( Kopete::Account* _account, const QString &name, Kopete::MetaContact *parent, Type type )
: Kopete::Contact( _account, name, parent )
{

//	qDebug() << "constructor of irccontact " << contactId();

	this->type = type;

	m_msgManager = 0L;

	if (type==Server) {
		status = IRCProtocol::OnlineServer;
		setOnlineStatus( IRCProtocol::protocol()->StatusOnline);
	} else if (type==Channel) {
		status = IRCProtocol::OnlineChannel;
		setOnlineStatus( IRCProtocol::protocol()->StatusChannel);
	} else {
		status = IRCProtocol::Online;
		setOnlineStatus( IRCProtocol::protocol()->StatusOnline );
	}

	setNickName(name);
}

IRCContact::~IRCContact()
{
//qDebug() << "destructor of irccontact " << contactId();
}

void IRCContact::setIRCStatus(IRCProtocol::IRCStatus status){
	setOnlineStatus(IRCProtocol::protocol()->statusLookup(status));
}

void IRCContact::serialize( QMap< QString, QString > &serializedData, QMap< QString, QString > & /* addressBookData */ )
{
    QString value;
	switch ( type )
	{
	case Server:
		value = QLatin1String("Server");
	case Channel:
		value = QLatin1String("Channel");
	case User:
		value = QLatin1String("User");
	}
	serializedData[ "contactType" ] = value;
}

Kopete::ChatSession* IRCContact::manager( CanCreateFlags canCreateFlags )
{

	if ( m_msgManager )
	{
		return m_msgManager;
	}
	else if ( canCreateFlags == CanCreate )
	{
		kDebug(14120) << "create manager for contact" << contactId() << endl;
		IRCAccount* account = static_cast<IRCAccount *> (this->account());

		QList<Kopete::Contact*> contacts;
		contacts.append(this);
		Kopete::ChatSession::Form form = ( type == Channel ?
				  Kopete::ChatSession::Chatroom : Kopete::ChatSession::Small );
		m_msgManager = Kopete::ChatSessionManager::self()->create(this->account()->myself(), contacts, protocol(), form );
		m_msgManager->setDisplayName(nickName());
		connect(m_msgManager, SIGNAL(messageSent(Kopete::Message&, Kopete::ChatSession*)),
			account, SLOT( sendMessage( Kopete::Message& , Kopete::ChatSession*) ) );
		connect(m_msgManager, SIGNAL(closing(Kopete::ChatSession*)),
			this, SLOT(slotChatSessionDestroyed(Kopete::ChatSession*)));
		return m_msgManager;
	}
	else
	{
		return 0;
	}
}


void IRCContact::showContactSettings()
{
}


//
//void IRCContact::receivedMessage( const QString &message )
//{
//	Kopete::ContactPtrList contactList;
//	contactList.append( account()->myself() );
//	// Create a Kopete::Message
//	Kopete::Message newMessage( this, contactList );
//	newMessage.setPlainBody( message );
//	newMessage.setDirection( Kopete::Message::Inbound );
//
//	// Add it to the manager
//	manager(CanCreate)->appendMessage (newMessage);
//}

void IRCContact::slotChatSessionDestroyed(Kopete::ChatSession* session) {
	m_msgManager = 0L;
	if (type==Channel)
		static_cast<IRCAccount*>(account())->slotChatSessionClose(session, this);
}


#include "irccontact.moc"

// vim: set noet ts=4 sts=4 sw=4:

