/*
	IRCcontact.h - Kopete IRC Protocol

	Copyright (c) 2009      by Volker Härtel		 <folkyvolk@gmx.de>
    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This library is free software; you can redistribute it and/or         *
    * modify it under the terms of the GNU General Public                   *
    * License as published by the Free Software Foundation; either          *
    * version 2 of the License, or (at your option) any later version.      *
    *                                                                       *
    *************************************************************************
*/

#ifndef IRCCONTACT_H
#define IRCCONTACT_H

#include <qmap.h>
#include <QList>
#include "kopetecontact.h"
#include "kopetemessage.h"
#include "ircprotocol.h"

class KAction;
class KActionCollection;
namespace Kopete { class Account; }
namespace Kopete { class ChatSession; }
namespace Kopete { class MetaContact; }

/**
@author Volker Härtel
*/
class IRCContact : public Kopete::Contact
{
	Q_OBJECT
public:
	/**
	 * The range of possible contact types
	 */
	enum Type { Server, Channel, User };


	IRCContact( Kopete::Account* _account, const QString &uniqueName,
			Kopete::MetaContact *parent, Type type = User );

	~IRCContact();

	/**
	 * Serialize the contact's data into a key-value map
	 * suitable for writing to a file
	 */
    virtual void serialize(QMap< QString, QString >& serializedData,
			QMap< QString, QString >& addressBookData);
	/**
	 * Returns a Kopete::ChatSession associated with this contact
	 */
	virtual Kopete::ChatSession *manager( CanCreateFlags canCreate = CanCreate );

	void setIRCStatus(IRCProtocol::IRCStatus status);

	Type type;

	IRCProtocol::IRCStatus status;

public slots:
	/**
	 * Transmits an outgoing message to the server 
	 * Called when the chat window send button has been pressed
	 * (in response to the relevant Kopete::ChatSession signal)
	 */
//	void sendMessage( Kopete::Message &message );
	/**
	 * Called when an incoming message arrived
	 * This displays it in the chatwindow
	 */
//	void receivedMessage( const QString &message );

	void slotChatSessionDestroyed(Kopete::ChatSession* session);

protected slots:
	/**
	 * Show the settings dialog
	 */
	void showContactSettings();


	
protected:



	Kopete::ChatSession* m_msgManager;
	KActionCollection* m_actionCollection;

	KAction* m_actionPrefs;
};

#endif
