/*
	IRCprotocol.cpp - Kopete IRC Protocol

	Copyright (c) 2009      by Volker Härtel		 <folkyvolk@gmx.de>
    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This library is free software; you can redistribute it and/or         *
    * modify it under the terms of the GNU Lesser General Public            *
    * License as published by the Free Software Foundation; either          *
    * version 2 of the License, or (at your option) any later version.      *
    *                                                                       *
    *************************************************************************
*/

#include <kgenericfactory.h>
#include <kdebug.h>

#include <kopete/kopeteaccountmanager.h>

#include "ircaccount.h"
#include "irccontact.h"
#include "ircprotocol.h"
#include "irceditaccountwidget.h"
#include "networkconfigwidget.h"

K_PLUGIN_FACTORY( IRCProtocolFactory, registerPlugin<IRCProtocol>(); )
K_EXPORT_PLUGIN( IRCProtocolFactory( "kopete_irc" ) )

IRCProtocol *IRCProtocol::s_protocol = 0L;

IRCProtocol::IRCProtocol( QObject* parent, const QVariantList &/*args*/ )
	: Kopete::Protocol( IRCProtocolFactory::componentData(), parent ),

	StatusOnline(  Kopete::OnlineStatus::Online, 25, this, Online,  QStringList(),
		  i18n( "Online" ),   i18n( "O&nline" ), Kopete::OnlineStatusManager::Online ),

	StatusOffline(  Kopete::OnlineStatus::Offline, 20, this, Offline,  QStringList(),
		  i18n( "Offline" ),   i18n( "O&ffline" ), Kopete::OnlineStatusManager::Offline ),

	StatusOp(  Kopete::OnlineStatus::Online, 50, this, Online | Operator,  QStringList("irc_op"),  i18n( "Operator" )),

	StatusVoice(  Kopete::OnlineStatus::Online, 40, this, Online | Voiced, QStringList("irc_voice"),  i18n( "Voice" ) ),

	StatusChannel(  Kopete::OnlineStatus::Online, 80, this, OnlineChannel, QStringList("irc_channel"),  i18n( "Channel" )),

	StatusOpVoice(Kopete::OnlineStatus::Online, 60, this, Online | Operator | Voiced, QString("irc_voice,irc_op").split(','), i18n("Operator")),

	StatusOpVoiceAway(Kopete::OnlineStatus::Away, 55, this, Online | Operator | Voiced | Away ,
			QString("irc_voice irc_op contact_away_overlay").split(' '), i18n("Away")),

	StatusChannelOffline(Kopete::OnlineStatus::Online,
			70, this, OfflineChannel, QStringList("irc_channel"), i18n("Offline")),

	StatusOpAway(Kopete::OnlineStatus::Away, 45, this, Online | Operator | Away,
			QString("irc_op contact_away_overlay").split(' '), i18n("Away")),

	StatusVoiceAway(Kopete::OnlineStatus::Away,	35, this, Online | Voiced | Away,
			QString("irc_voice contact_away_overlay").split(' '),  i18n("Away")),

	StatusAway(Kopete::OnlineStatus::Away, 2, this, Online | Away, QStringList("contact_away_overlay"),
			i18n("Away"), i18n("Away"), Kopete::OnlineStatusManager::Away),

	StatusConnecting(Kopete::OnlineStatus::Connecting, 1, this, Connecting,
					 QStringList("irc_connecting"), i18n("Connecting"))

{
	kDebug( 14210 ) ;


	s_protocol = this;
}

IRCProtocol::~IRCProtocol()
{
	qDebug() << "irc protocol is deleted";
}

Kopete::Contact *IRCProtocol::deserializeContact(
	Kopete::MetaContact *metaContact, const QMap<QString, QString> &serializedData,
	const QMap<QString, QString> &/* addressBookData */)
{
	QString contactId = serializedData[ "contactId" ];
	QString accountId = serializedData[ "accountId" ];
//	QString displayName = serializedData[ "displayName" ];
	QString type = serializedData[ "contactType" ];

	IRCContact::Type tbcType;
	if ( type == QLatin1String( "Server" ) )
		tbcType = IRCContact::Server;
	else if ( type == QLatin1String( "Channel" ) )
		tbcType = IRCContact::Channel;
	else if ( type == QLatin1String( "User" ) )
		tbcType = IRCContact::User;
	else
		tbcType = IRCContact::User;

	QList<Kopete::Account*> accounts = Kopete::AccountManager::self()->accounts( this );
	Kopete::Account* account = 0;
	foreach( Kopete::Account* acct, accounts )
	{
		if ( acct->accountId() == accountId )
			account = acct;
	}

	if ( !account )
	{
		kDebug(14210) << "Account doesn't exist, skipping";
		return 0;
	}

	IRCContact * contact = new IRCContact(account, contactId, metaContact, tbcType);

	return contact;
}

AddContactPage * IRCProtocol::createAddContactWidget( QWidget *parent, Kopete::Account * )
{
	kDebug( 14210 ) << "Creating Add Contact Page";
	return 0;
//	return new IRCAddContactPage( parent );
}

KopeteEditAccountWidget * IRCProtocol::createEditAccountWidget( Kopete::Account *account, QWidget *parent )
{
	kDebug( 14210 ) << "Creating Edit Account Page" << endl;
	return new IRCEditAccountWidget(static_cast<IRCAccount*>(account) ,parent );
}

Kopete::Account *IRCProtocol::createNewAccount( const QString &accountId )
{
	return new IRCAccount( this, accountId );
}

IRCProtocol *IRCProtocol::protocol()
{
	return s_protocol;
}

void IRCProtocol::editNetworks(const QString &networkName, QWidget *parent)
{

	IRCNetworkConfigWidget *netConf = new IRCNetworkConfigWidget(parent );
	netConf->setModal(true);
	netConf->setAttribute( Qt::WA_DeleteOnClose );
	netConf->editNetworks(networkName);
	netConf->show();
}


const Kopete::OnlineStatus IRCProtocol::statusLookup( IRCStatus status ) const
{
	switch( status )
	{
	case Offline:
		return StatusOffline;
	case Connecting:
		return StatusConnecting;

	// Regular user
	case Online:
		return StatusOnline;
	case Online | Away:
		return StatusAway;

	// Voiced
	case Online | Voiced:
		return StatusVoice;
	case Online | Away | Voiced:
		return StatusVoiceAway;

	// Operator
	case Online | Operator:
		return StatusOp;
	case Online | Away | Operator:
		return StatusOpAway;
	case Online | Operator | Voiced:
		return StatusOpVoice;
	case Online | Operator | Voiced | Away:
		return StatusOpVoiceAway;

	// Server
	case OnlineServer:
		return StatusOnline;
	case OfflineServer:
		return StatusOffline;

	// Channel
	case OnlineChannel:
		return StatusChannel;
	case OfflineChannel:
		return StatusChannelOffline;

	default:
		return StatusOnline;
	}
}


#include "ircprotocol.moc"
