/*
	IRCprotocol.h - Kopete IRC Protocol

	Copyright (c) 2009      by Volker Härtel		 <folkyvolk@gmx.de>
    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This library is free software; you can redistribute it and/or         *
    * modify it under the terms of the GNU General Public                   *
    * License as published by the Free Software Foundation; either          *
    * version 2 of the License, or (at your option) any later version.      *
    *                                                                       *
    *************************************************************************
*/

#ifndef IRCPROTOCOL_H
#define IRCPROTOCOL_H

#include <kopeteprotocol.h>
#include "kopeteuiglobal.h"


/**
 * Encapsulates the generic actions associated with this protocol
 * @author Volker Härtel
 */
class IRCProtocol : public Kopete::Protocol
{
	Q_OBJECT
public:


	enum IRCStatus
	{
		Offline        = 1,                 //! An offline user.
		Connecting     = 2,                 //! User that is connecting.
		Away           = 4,                 //! User that is away. May be regular user, voiced user or (server) operator.
		Online         = 8,                 //! This user is online.
		Voiced         = 16,                //! This user is voiced.
		Operator       = 32,                //! This user is a channel operator.
		ServerOperator = 1024,              //! This user is a server operator.
		OfflineChannel = 4096,              //! This channel is offline.
		OnlineChannel  = 8192,              //! This channel is online.
		OfflineServer  = 16384,             //! This server is offline.
		OnlineServer   = 32768              //! This server is online.
	};


	IRCProtocol(QObject *parent, const QVariantList &args);

	~IRCProtocol();

	/**
	 * Convert the serialised data back into a IRCContact and add this
	 * to its Kopete::MetaContact
	 */
	virtual Kopete::Contact *deserializeContact(
			Kopete::MetaContact *metaContact,
			const QMap< QString, QString > & serializedData,
			const QMap< QString, QString > & addressBookData
		);

	/**
	 * Generate the widget needed to add IRCContacts
	 */
	virtual AddContactPage * createAddContactWidget( QWidget *parent, Kopete::Account *account );

	/**
	 * Generate the widget needed to add/edit accounts for this protocol
	 */
	virtual KopeteEditAccountWidget * createEditAccountWidget( Kopete::Account *account, QWidget *parent );

	/**
	 * Generate a IRCAccount
	 */
	virtual Kopete::Account * createNewAccount( const QString &accountId );

	/**
	 * Access the instance of this protocol
	 */
	static IRCProtocol *protocol();

	const Kopete::OnlineStatus statusLookup( IRCStatus status ) const;

	const Kopete::OnlineStatus StatusOnline;
	const Kopete::OnlineStatus StatusOffline;
	const Kopete::OnlineStatus StatusOp;
	const Kopete::OnlineStatus StatusVoice;
	const Kopete::OnlineStatus StatusVoiceAway;
	const Kopete::OnlineStatus StatusOpAway;
	const Kopete::OnlineStatus StatusChannel;
	const Kopete::OnlineStatus StatusChannelOffline;
	const Kopete::OnlineStatus StatusAway;
	const Kopete::OnlineStatus StatusOpVoice;
	const Kopete::OnlineStatus StatusOpVoiceAway;
	const Kopete::OnlineStatus StatusConnecting;

public slots:
	void editNetworks(const QString &networkName, QWidget *parent = Kopete::UI::Global::mainWidget());

protected:
	static IRCProtocol *s_protocol;


};

#endif
